import os
import logging
from paste.script import command
from paste.deploy import appconfig

from kid.model import indexer

log = logging.getLogger('kid')
if not log.handlers:
    log.addHandler(logging.StreamHandler())

class GenerateIndexs(command.Command):

    max_args = 1
    min_args = 1
    takes_config_file = 1

    usage = "config_file.ini"
    summary = "Generate Indices from Reuters Dataset"
    group_name = "Indexing"

    parser = command.Command.standard_parser(verbose=True, interactive=True)

    def command(self):
        if self.verbose:
            log.setLevel(logging.DEBUG)
        try:
            conf = appconfig('config:%s#main' %(os.path.abspath(self.args[0])))
        except Exception, inst:
            raise command.BadCommand('Error reading config file %s [%s]' %(self.args[0], inst))
        params = ['dataset', 'index.forward', 'index.inverted']
        values = []
        for key in params: 
            setting = 'kid.%s' %key
            if setting not in conf.local_conf:
                raise command.BadCommand(
                    'Error %s not set in app:main section of config file' %(setting)
                )
            values.append(conf.local_conf[setting])        
        reuters, forward, inverted = values
        try:
            reuters_file = open(reuters)
            if not (
                reuters_file.readline().startswith('<?xml') and
                reuters_file.readline().startswith('<newsitem')):
                raise command.BadCommand('%s does not seem to be a reuters dataset' %(reuters))
            reuters_file.close()
        except command.BadCommand:
            reuters_file.close()
            raise
        except Exception, inst:
            raise command.BadCommand('Cannot open Dataset at %s [%s]' %(reuters, inst))
        for dbfile in forward,inverted:
            if os.path.exists(dbfile) and self.ask('Delete exisitng %s' %dbfile, default='careful'):
                os.unlink(dbfile)
            self.ensure_dir(os.path.dirname(dbfile))
        log.debug("Parsing %s to generate %s and %s", reuters, forward, inverted)
        indexer.generate_indexs(reuters, forward, inverted)
        log.debug("Indices successfully created")
