import logging

from pylons import request, response, session, config
from pylons import tmpl_context as c
from pylons.controllers.util import abort, redirect_to

from kid.lib.base import BaseController, render, h

import formencode
import kid.model as model
import kid.model.indexer as indexer

from urllib import quote_plus, unquote_plus

log = logging.getLogger(__name__)
results_per_page = int(config['kid.search.results_per_page'])


class SearchController(BaseController):

    def index(self, query="", id=1):
        def parse(q):
            return indexer.splitter.split(
                indexer.punct.sub('',
                    indexer.tags.sub(' ', q)
            ))
        params = {}
        if request.params:
            params = dict(request.params)
        elif query and parse(query) != self.cursor.query:
            params = {'query': unquote_plus(query)}
        if params:
            schema = model.QueryController()
            try:
                form_result = schema.to_python(params)
                queryp = parse(form_result['query'])
                self.cursor.execute(queryp)
                session['query_parts'] = queryp
                session.save()
                if request.params:
                    redirect_to(query=quote_plus(form_result['query']))
            except formencode.validators.Invalid, error:
                for k,v in error.error_dict.iteritems():
                    error.error_dict[k] = str(v).encode('us-ascii')
                c.errors = error.error_dict
        page = id
        c.query = unquote_plus(query)
        if c.query:
            c.stop_words = indexer.stop_words
            if 'query_parts' in session:
                c.query_parts = session['query_parts']
            if self.cursor.time:
              c.time = self.cursor.time
              c.not_in_index = self.cursor.unindexed
              c.records = h.paginate.Page(
                  self.cursor, page, items_per_page=results_per_page
              )
        c.reserved = indexer.reserved_words
        return render('/main.mako')

    def document(self, id, length):
        '''
        Returns the document text given the offset `id` and byte `length`
        '''
        id, length = [int(x) for x in id, length]
        fakedoc = indexer.Document(offset=id, length=length)
        response.headers['content-type'] = 'text/xml; charset=utf-8'
        return model.QE.retrieve_doctext(fakedoc)

    def help(self, id):
        '''
        Returns the appropriate help section `id`
        '''
        c.titles = {
            "about": "About KiD",
            "parser": "Query Parser",
            "status": "Query Status Bar"
        }
        setattr(c, id, True)
        if id != "all" and id not in c.titles:
            abort(404, 'Check Your URL')
        return render('/help.mako')

    def _get_cursor(self):
        if 'cursor' not in session or \
            not indexer.is_cursor(session['cursor']):
            session['cursor'] = indexer.get_cursor(model.QE)
            session.save()
        return indexer.get_cursor(session['cursor'])
    cursor = property(_get_cursor)
